use {
    crate::prelude::{IdError, ParseMessageError, ParseRequestError},
    std::{fmt, io, string::FromUtf8Error},
};

#[derive(Debug)]
pub enum Error {
    Message(ParseMessageError),
    Id(IdError),
    Io(io::Error),
    Request(ParseRequestError),
    Storage(String),
    Rustls(rustls::Error),
    Utf8,
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Message(e) => write!(f, "{e}"),
            Self::Id(e) => write!(f, "{e}"),
            Self::Io(e) => write!(f, "{e}"),
            Self::Storage(e) => write!(f, "{e}"),
            Self::Request(e) => write!(f, "{e}"),
            Self::Rustls(e) => write!(f, "{e}"),
            Self::Utf8 => write!(f, "Utf8 error"),
        }
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Message(e) => Some(e),
            Self::Id(e) => Some(e),
            Self::Io(e) => Some(e),
            Self::Request(e) => Some(e),
            Self::Rustls(e) => Some(e),
            _ => None,
        }
    }
}

impl From<IdError> for Error {
    fn from(value: IdError) -> Self {
        Self::Id(value)
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

impl From<ParseMessageError> for Error {
    fn from(value: ParseMessageError) -> Self {
        Self::Message(value)
    }
}

impl From<ParseRequestError> for Error {
    fn from(value: ParseRequestError) -> Self {
        Self::Request(value)
    }
}

impl From<rustls::Error> for Error {
    fn from(value: rustls::Error) -> Self {
        Self::Rustls(value)
    }
}

impl From<FromUtf8Error> for Error {
    fn from(_value: FromUtf8Error) -> Self {
        Self::Utf8
    }
}
