use crate::{fingerprint::GetFingerprint, mailuser::Mailuser};
use rustls::server::{ClientCertVerified, ClientCertVerifier};
use std::{sync::{Arc, Mutex}, io::Read};
use x509_parser::prelude::*;

#[derive(Debug)]
pub struct Verifier<S: FingerPrintStore> {
    pub store: Mutex<S>,
}

pub trait FingerPrintStore: Send + Sync {
    fn get_mailuser(&self, fingerprint: &str) -> Option<Mailuser>;
    fn insert_mailuser(&mut self, fingerprint: &str, user: &str) -> Option<Mailuser>;
    fn contains_mailuser(&self, fingerprint: &str) -> bool;
}

impl<S: FingerPrintStore> ClientCertVerifier for Verifier<S> {
    fn client_auth_root_subjects(&self) -> &[rustls::DistinguishedName] {
        todo!()
    }

    fn verify_client_cert(
        &self,
        end_entity: &rustls::Certificate,
        intermediates: &[rustls::Certificate],
        now: std::time::SystemTime,
    ) -> Result<ClientCertVerified, rustls::Error> {
        let fingerprint = end_entity.fingerprint()?;
        if let Ok(store) = self.store.lock() {
            if let Some(user) = store.get_mailuser(&fingerprint.fingerprint) {
                let (_, pk) = X509Certificate::from_der(end_entity.as_ref()).map_err(|e| {
                    rustls::Error::InvalidCertificate(rustls::CertificateError::Other(Arc::new(e)))
                })?;
                let subject = pk.subject();
                let mut name_match = false;
                subject.iter_common_name().for_each(|n| {
                    let mut val = n.attr_value().data;
                    let mut name = String::new();
                    if val.read_to_string(&mut name).is_ok() {
                        name_match = name == user.to_string();
                    }
                });
            }
        }
        todo!()
    }
}
